function [s,latitude,longitude,f,majcoords,sigma] = fit_ellipse(lat,lon,z,con,plotit)

% lat/lon of ozone contour
c=contourc(double(lon),double(lat),double(z'),[con,con]);
c=c(:,c(2,:)<-45); %
c=c(:,(find(c(1,:)==0)+1):end);

if isempty(c) || length(c(1,:)) < (0.6*length(lon))
    s=NaN(1,6);
    latitude=[];
    longitude=[];
    f=NaN;
    majcoords=NaN(1,2);
    sigma = NaN(1,4);
    fprintf(' - no contour %i  %i\n',con,plotit+1000) 
    return
end

if plotit>1500
    mapit_notoolbox(lat,lon,z,0,'',0)
    hold on; plot(c(1,:),c(2,:),'w','Linewidth',4)
    title(num2str(plotit))
end

% convert to cartesian
x=cos(c(1,:)*pi/180.0).*cos(c(2,:)*pi/180.0) ...
    ./ (1.0-sin(c(2,:)*pi/180.0));

y=-sin(c(1,:)*pi/180.0).*cos(c(2,:)*pi/180.0) ...
    ./ (1.0-sin(c(2,:)*pi/180.0));

% figure;
% plot(x,y)

% fit ellipse
param=EllipseFitByTaubin([x;y]');

% Convert the parameters derived using the Taubin algorithm
% to elliptical parameters. The conversion
% process is indicated on this webpage
% See http://www.mathworks.com/matlabcentral/fileexchange/22683

e_data =ellipse_conversion(param);
x0 =e_data.x0;
y0 =e_data.y0;

% check major/minor axes
if(e_data.major_length>e_data.minor_length)
    major =e_data.major_length;
    minor =e_data.minor_length;
    m_theta =e_data.theta;
    % this function makes an array with the x, y positions
    % of the ellipse
    [x1,y1]=ellipse_function(e_data.x0,e_data.y0,e_data.major_length,e_data.minor_length,e_data.theta);
else
    minor =e_data.major_length;
    major =e_data.minor_length;
    m_theta =e_data.theta+pi/2.0;
    % this function makes an array with the x, y positions
    % of the ellipse
    [x1,y1]=ellipse_function(e_data.x0,e_data.y0,e_data.minor_length,e_data.major_length,e_data.theta+pi/2.0);
end

ecc_xy = sqrt(1-(minor/major)^2);
[sigma_x0,sigma_y0,sigma_ecc,sigma_ell] = ellipse_error(x',y',param,x0,y0,ecc_xy);
sigma = [sigma_x0,sigma_y0,sigma_ecc,sigma_ell];

if isreal(x1) && isreal(y1)
    % covert  back to lat/lon
    longitude=mod(-atan2(y1,x1)/pi*180,360);
    latitude=asin((-2./(x1.^2+y1.^2+1))+1)/pi*180;
    
    if plotit>1500
        hold on ; plot(longitude,latitude,'m','LineWidth',2)
    end
    
    m_longitude0 =mod(-atan2(y0 ,x0 )*(180.0/pi),360);
    m_latitude0 =asin((-2./(x0.^2+y0.^2+1))+1)*(180.0/pi);
    
    % lengths in km
    for jj=1:180
        lat1=m_latitude0 ;
        long1=m_longitude0 ;
        lat2=latitude(jj);
        long2=longitude(jj);
        seperation(jj)=distance_between_points(lat1,long1,lat2,long2);
    end
    m_minor_axis =min(seperation);
    [m_major_axis,maxidx] =max(seperation);
    majcoords=[latitude(maxidx),longitude(maxidx)];
    Ell_Area =pi*m_minor_axis *m_major_axis ;
    Ell_Ecc =sqrt(1-(m_minor_axis /m_major_axis )^2);
    
    if plotit>1500
        hold on ; plot(m_longitude0,m_latitude0,'mo')
        ylim([-90,-35])
        %title(['ecc = ',num2str(ecc_xy,'%4.3f'),'  sigma_ecc = ',num2str(sigma_ecc,'%4.3f')])
    end
    
    s=[m_theta/pi*180,m_latitude0,m_longitude0,Ell_Ecc,Ell_Area,(m_major_axis-m_minor_axis)/m_major_axis];
    
    f = frac_overlap(latitude,longitude,c(2,:),c(1,:));
else
    s=NaN(1,6);
    latitude=[];
    longitude=[];
    majcoords=NaN(1,2);
    f=NaN;
    fprintf(' - no fit\n')
end